import { UDF_LIST } from '@/config/UDF_LIST';
import { Button, Dialog } from '@/lib/components';
import { useActiveQuery, useAppDispatch } from '@/lib/data-access';
import { setQuerybuilderGraph } from '@/lib/data-access/store/sessionSlice';
import { SchemaReactflowMetaNode } from '@/lib/querybuilder/model';
import { MetaPillClickType, ProceduresPanel } from '@/lib/querybuilder/panel/procedures/ProceduresDialog';
import { cloneDeep } from 'lodash-es';
import React, { useRef, useState, useEffect } from 'react';
import { Node } from 'reactflow';
import { MetaPillUDFData, NumberFilterTypes } from 'ts-common';

export const QueryMetaPill = React.memo((node: SchemaReactflowMetaNode) => {
  const [isMenuVisible, setIsMenuVisible] = useState(false);
  const [size, setSize] = useState({ width: node.data.width ?? 400, height: node.data.height ?? 200 });
  const [isHovered, setIsHovered] = useState(false);
  const activeQuery = useActiveQuery();
  const dispatch = useAppDispatch();

  useEffect(() => {
    setSize({ width: node.data.width ?? 400, height: node.data.height ?? 200 });
  }, [node.data.width, node.data.height]);

  // TODO implement correct classification
  const classifySelection = () => {
    return 'Path';
  };

  // filters list of udfs based on classification
  const relevantUdfs = UDF_LIST.filter(udf => udf.input === classifySelection());

  const resizingRef = useRef(false);

  const handleClick = (event: React.MouseEvent) => {
    event.preventDefault();
    setIsMenuVisible(true);
  };

  const handleClose = () => {
    setIsMenuVisible(false);
  };

  const handleMouseDown = (event: React.MouseEvent) => {
    event.preventDefault();
    event.stopPropagation();
    resizingRef.current = true;
    // if (props.setDragging) props.setDragging(false); // Disable React Flow drag
    document.addEventListener('mousemove', handleMouseMove);
    document.addEventListener('mouseup', handleMouseUp);
  };

  const handleMouseMove = (event: MouseEvent) => {
    if (!resizingRef.current) return;
    setSize(prevSize => ({
      width: Math.max(100, prevSize.width + event.movementX),
      height: Math.max(100, prevSize.height + event.movementY),
    }));
  };

  const handleMouseUp = () => {
    resizingRef.current = false;
    // if (props.setDragging) props.setDragging(false); // Disable React Flow drag
    document.removeEventListener('mousemove', handleMouseMove);
    document.removeEventListener('mouseup', handleMouseUp);
  };

  const handleDragEnter = (event: React.DragEvent) => {
    event.preventDefault();
    setIsHovered(true);
  };

  const handleDragLeave = (event: React.DragEvent) => {
    event.preventDefault();
    setIsHovered(false);
  };

  const handleDrop = (event: React.DragEvent) => {
    event.preventDefault();
    setIsHovered(false);
    const draggedItem = event.dataTransfer.getData('application/reactflow');

    // if (draggedItem && props.setSelection) {
    //   const parsedItem = JSON.parse(draggedItem);
    //   props.setSelection([...(props.selection ?? []), parsedItem]); // Update selection state
    // }
  };



  const onClick = (type: MetaPillClickType, config: { op: string; right: number } = { op: NumberFilterTypes.LESS_THAN, right: 2 }) => {
    if (!activeQuery) return;

    if (type === 'degreeleft') {
      // Existing degreeleft logic
      const nodes: Node[] = node.data.selection.filter((node: Node) => node.type === 'entity');
      const nodeToLeft = nodes.find(graphNodes => !activeQuery.graph.edges.some(edge => edge.target === graphNodes.id));
      if (!nodeToLeft) return;
      const nodeToRight = nodes.find(graphNodes => !activeQuery.graph.edges.some(edge => edge.source === graphNodes.id));

      console.log('degreeleft', node);
      const graph = cloneDeep(activeQuery.graph);

      graph.nodes.forEach(graphNode => {
        console.log(node, graphNode);
        if (node.id === graphNode.key) {
          graphNode.attributes = {
            ...graphNode.attributes,
            udf: [
              {
                comparison: true,
                type: 'degree',
                side: 'left',
                leftId: nodeToLeft.id,
                rightId: nodeToRight?.id,
                op: config.op,
                right: config.right,
              },
            ],
          };
        }
      });

      dispatch(setQuerybuilderGraph(graph));
    }
    else if (type === 'community_detection') {
      const graph = cloneDeep(activeQuery.graph);

      graph.nodes.forEach(graphNode => {
        if (node.id === graphNode.key) {
          graphNode.attributes = {
            ...graphNode.attributes,
            udf: [
              {
                comparison: false,
                type: 'community_detection',
              },
            ],
          };
        }
      });

      console.log("community detection graph:", graph);
      dispatch(setQuerybuilderGraph(graph));
    }
  };
  const getEntityToSide = (udfData: MetaPillUDFData): string => {
    if (!activeQuery?.graph.edges) return '';
    return activeQuery.graph.nodes.find(node => node.key === udfData.leftId)?.attributes?.name ?? '';
  };

  const handleDelete = () => {
    // delete meta pill
    if (!activeQuery) return;
    const graph = cloneDeep(activeQuery.graph);
    // graph.nodes = graph.nodes.filter(graphNode => graphNode.key !== node.id);
    // graph.edges = graph.edges.filter(edge => edge.source !== node.id && edge.target !== node.id);
    // console.log(graph);
    dispatch(setQuerybuilderGraph(graph));
  };

  return (
    <div className="w-fit h-fit nowheel">
      <div
        className="meta-pill flex items-end justify-end relative border-2 border-[#c8cbd0] rounded cursor-grab"
        onDragEnter={handleDragEnter}
        onDragLeave={handleDragLeave}
        onDragOver={e => e.preventDefault()}
        onDrop={handleDrop}
        style={{
          width: size.width,
          height: size.height,
          zIndex: 1,
        }}
      >
        <div className="absolute top-[-25px] right-0 flex">
          <Button variantType="secondary" onClick={handleClick} variant="ghost" size="xs" iconComponent="icon-[ic--baseline-settings]" />
          <Button variantType="secondary" variant="ghost" size="xs" iconComponent="icon-[mdi--trash-outline]" onClick={handleDelete} />
        </div>


        <div
          className="absolute bottom-[-1.75rem] right-0 w-full flex justify-end px-6 pb-1 rounded-b-md text-sm border border-[#e4e5e7] bg-[#f4f5f6]"
          style={{ width: node.data.width - 5 }}
        >

          {node.data.udf &&
            node.data.udf.length > 0 &&
            node.data.udf.map((udfItem, index) => (
              <span key={index}>
                {udfItem.type}({getEntityToSide(udfItem)})
                {udfItem.comparison ? ` ${udfItem.op} ${udfItem.right}` : ""}
              </span>


            ))}
        </div>
      </div>

      <Dialog open={isMenuVisible} onOpenChange={handleClose}>
        <ProceduresPanel udfList={relevantUdfs} selection={[]} handleClose={handleClose} onClick={onClick} />
      </Dialog>
    </div>
  );
});
